"""This module contains Requirements class, which is the core of the current process."""

import re
from typing import List, Tuple


class Requirements:
    """
    A class that handles parsing and rendering requirements data.

    Attributes:
        None

    Methods:
        _read_file_contents: Read the contents of a file.
        _from_pip_freeze: Parse data in the pip freeze format.
        _from_pip_list: Parse data in the pip list format.
        render: Render the requirements data based on the specified format.
    """

    def _read_file_contents(self, source_path: str) -> str:
        """
        Read the contents of a file.

        Args:
            source_path: The path to the file to be read.

        Returns:
            The contents of the file as a string.
        """
        return open(source_path, "r").read()

    def _from_pip_freeze(self, data: str) -> List[Tuple[str, str]]:
        """
        Parse data in the pip freeze format.

        Args:
            data: The pip freeze formatted data to be parsed.

        Returns:
            A list of tuples containing package names and versions.
        """
        lines = data.strip().split("\n")
        pattern = r"(==|<=|>=|<|>)"
        package_data = [re.split(pattern, line) for line in lines]
        return [
            (package[0], package[2]) if len(package) > 1 else package
            for package in package_data
        ]

    def _from_pip_list(self, data: str) -> List[Tuple[str, str]]:
        """
        Parse data in the pip list format.

        Args:
            data: The pip list formatted data to be parsed.

        Returns:
            A list of tuples containing package names and versions.
        """
        lines = data.strip().split("\n")
        package_data = [tuple(line.split()) for line in lines[2:]]
        return [(package[0], package[1]) for package in package_data]  # pylint

    def render(self, source_path: str, format: str) -> List[Tuple[str, str]]:
        """
        Render the requirements data based on the specified format.

        Args:
            source_path: The path to the requirements file.
            format: The format of the requirements file (e.g., "pip_freeze", "pip_list"). # noqa

        Returns:
            A list of tuples containing package names and versions.
        """
        data = self._read_file_contents(source_path)
        if format == "pip_freeze":
            return self._from_pip_freeze(data)
        if format == "pip_list":
            return self._from_pip_list(data)



import re

from extractor.logger import logger


class StandardCheck:
    def gh_pattern(self, pattern: str, url: str, custom_error: str = None):
        try:
            found = re.search(pattern, url)
        except Exception as e:
            logger.warning(custom_error) if custom_error else logger.warning(e)
        else:
            return found

    def additional_urls(self, pattern: str, project_urls: str):
        urls = project_urls.items()
        possible_keys = [
            "Code",
            "Source Code",
            "Source code",
            "Source",
            "Homepage",
            "Home",
            "Repository",
            "repository",
        ]
        for title, url in urls:
            if _ := self.gh_pattern(pattern, url) and title in possible_keys:
                return url

    def version(
        self, version: str, pattern: str, raw_data: dict, filtered_data: dict
    ) -> dict:
        project_default_error = "No project url found, please check manually"
        version_default_error = "No version url found, please check manually"
        if version and self.gh_pattern(
            pattern, filtered_data.get("project_url"), project_default_error
        ):
            filtered_data[
                "version_url"
            ] = f"{filtered_data['project_url']}/tree/{version}/"
        elif raw_data["release_url"] and self.gh_pattern(
            pattern, filtered_data.get("project_url"), version_default_error
        ):
            filtered_data["version_url"] = raw_data["release_url"]

        else:
            filtered_data["version_url"] = version_default_error
            filtered_data["project_url"] = project_default_error
        return filtered_data



import pandas as pd
import toml
from rich import print
from rich.progress import track

from extractor.core import filter_data, get_raw_data
from extractor.logger import logger
from extractor.render import Requirements


def get_version_from_pyproject():
    with open("pyproject.toml") as f:
        pyproject_data = toml.load(f)
        version = (
            pyproject_data.get("tool", {}).get("poetry", {}).get("version")
        )  # noqa
    return version


def filter_requirements(requirements_file_data, installed_requirements_data):
    base_reqs = [req[0] for req in requirements_file_data]
    return [
        ireq for ireq in installed_requirements_data if ireq[0] in base_reqs
    ]  # noqa


def get_filtered_metadata(requirements_file, installed_requirements, output):
    handler = Requirements()
    print("Getting requirements list...")
    requirements_file_data = handler.render(requirements_file, "pip_freeze")
    print("Getting installed libraries list...")
    installed_requirements_data = handler.render(
        installed_requirements, "pip_freeze"
    )  # noqa

    result = filter_requirements(
        requirements_file_data, installed_requirements_data
    )  # noqa

    pkgs_raw_metadata = []
    for pkg in track(result, description="Processing..."):
        filtered_data = filter_data(get_raw_data(pkg[0]), pkg[1])
        pkgs_raw_metadata.append(filtered_data)
    df = pd.DataFrame(pkgs_raw_metadata)
    print(f"Storing into: {output}")
    if output.endswith(".csv"):
        df.to_csv(output, index=False)
    elif output.endswith(".xlsx"):
        df.to_excel(output, index=False)
    else:
        logger.error("Not supported format.")


if __name__ == "__main__":
    requirements_file = "./local/all.txt"
    installed_requirements = "./local/installed.txt"
    output = "./local/ece/metadata.xlsx"
    get_filtered_metadata(requirements_file, installed_requirements, output)



from pathlib import Path
from typing import Dict

import pandas as pd
import requests
from rich.progress import track

from extractor.checks import StandardCheck
from extractor.logger import logger
from extractor.render import Requirements

URLBASE = "https://pypi.org/pypi"


def get_raw_data(project: str) -> Dict[str, str]:
    """
    Retrieve raw metadata for a project from a given URL.

    Args:
        project: The name of the project.

    Returns:
        A dictionary containing the raw metadata of the project.
    """
    try:
        r = requests.get(
            f"{URLBASE}/{project}/json", headers={"Accept": "application/json"}
        )
    except Exception as e:
        logger.error(e)
    else:
        return r.json()["info"]


def filter_data(raw_data: Dict[str, str], version: str) -> Dict[str, str]:
    """
    Filter relevant metadata from raw data.

    Args:
        raw_data: The raw metadata of a project.
        version: The version of the project.

    Returns:
        A dictionary containing filtered metadata.
    """
    project_name = raw_data["name"]
    project_url = raw_data["project_url"]
    project_urls = raw_data["project_urls"]
    gh_url_pattern = r"(https:\/\/|http:\/\/)github\.com"
    filtered_data = {
        "name": project_name,
        "version": version or raw_data["version"],
        "license": raw_data["license"],
        "homepage": raw_data["home_page"],
        "release_url": raw_data["release_url"],
    }
    check = StandardCheck()
    logger.info(f"Searching GitHub url for: {project_name}")

    # logger.info(f"Searching GitHub url for: {project_name}")
    if project_url != "" and check.gh_pattern(gh_url_pattern, project_url):
        filtered_data["project_url"] = project_url

    if project_urls:
        logger.debug("Nested metadata found")
        filtered_data["project_url"] = check.additional_urls(
            gh_url_pattern, project_urls
        )
    if project_name == "pandas":
        version = f"v{filtered_data['version']}"
        filtered_data["version"] = version

    filtered_data = check.version(version, gh_url_pattern, raw_data, filtered_data)
    return filtered_data


def extract_data(source_path: Path, format: str) -> None:
    """
    Extract data based on the specified requirements format.

    Args:
        source_path: The path to the requirements file.
        format: The format of the requirements file.

    Returns:
        pd.DataFrame
    """
    logger.info("Starting process")
    logger.debug(f"Retrieving: {source_path}")
    result = Requirements().render(source_path, format)
    pkgs_raw_metadata = []
    for pkg in track(result):
        filtered_data = filter_data(
            get_raw_data(pkg[0]), pkg[1] if len(pkg) > 1 else None
        )
        pkgs_raw_metadata.append(filtered_data)
    return pd.DataFrame(pkgs_raw_metadata)


def save_data(data: pd.DataFrame, output: Path):
    logger.info(f"Storing into: {output}")
    if str(output).endswith(".csv"):
        data.to_csv(output, index=False)
        logger.info("All done! Have a Great day")
    elif str(output).endswith(".xlsx"):
        data.to_excel(output, index=False)
        logger.info("All done! Have a Great day")
    else:
        logger.error("Not supported format.")
